@@ -1,5 +1,6 @@
 import delayedarray
 import numpy
+import scipy.sparse
 
 
 def test_Transpose_simple():
@@ -16,6 +17,7 @@ def test_Transpose_simple():
     assert isinstance(t.seed, delayedarray.Transpose)
     assert t.shape == (23, 30)
     assert (numpy.array(t) == numpy.transpose(y)).all()
+    assert not delayedarray.is_sparse(t)
 
 
 def test_Transpose_more_dimensions():
@@ -33,6 +35,22 @@ def test_Transpose_more_dimensions():
     assert (numpy.array(t) == numpy.transpose(y)).all()
 
 
+def test_Transpose_subset():
+    y = numpy.random.rand(30, 23, 10)
+    x = delayedarray.DelayedArray(y)
+    t = numpy.transpose(x)
+
+    subset = (range(2, 8), range(3, 16), range(4, 24))
+    assert (delayedarray.extract_dense_array(t, subset) == y.T[numpy.ix_(*subset)]).all()
+
+
+def test_Transpose_sparse():
+    y = scipy.sparse.rand(30, 23)
+    x = delayedarray.DelayedArray(y)
+    t = numpy.transpose(x)
+    assert delayedarray.is_sparse(t)
+
+
 def test_Transpose_dask():
     y = numpy.random.rand(30, 23, 10)
     x = delayedarray.DelayedArray(y)
